#include <QPainter>
#include <QDebug>
#include <QTouchEvent>
#include <QRegion>
#include <QPalette>
#include "blobfingercount.h"

int BlobFingerCount::delayActivate = 300;

BlobFingerCount::BlobFingerCount(QWidget * _parent, bool _right) : QWidget(_parent,Qt::CustomizeWindowHint | Qt::WindowTitleHint | Qt::FramelessWindowHint ),m_right(_right) {
    setAttribute(Qt::WA_AcceptTouchEvents);
    //setAttribute(Qt::WA_TranslucentBackground);
    //setAttribute(Qt::WA_MouseNoMask);
    //setAutoFillBackground(false);
    //this->setAttribute(Qt::WA_NoSystemBackground);
    //this->setAttribute(Qt::WA_NoBackground);
    //this->setAttribute(Qt::WA_OpaquePaintEvent);
    //this->setMask(QRegion(0,0,1,1));

    setWindowOpacity(0.1);
    qDebug()<<"creation BlobFingerCountFingerCount";

    //QPalette pal = this->palette();
    //pal.setColor(this->backgroundRole(),Qt::transparent);
    //pal.setColor(this->backgroundRole(),QColor(0,0,0,200));
    //this->setPalette(pal);
    m_panTimer.setSingleShot(true);
    m_panTimer.setInterval(delayActivate);
    nbPreviousPoints = 0;
    //hide();
    connect(&m_panTimer,SIGNAL(timeout()), this, SLOT(panTimeout()));
}

//------------------
BlobFingerCount::~BlobFingerCount() {}

//------------------
void BlobFingerCount::paintEvent ( QPaintEvent *e)
{
    // QWidget::paintEvent(e);
    QPainter p(this);

    QColor c = Qt::blue;
    if (m_right) c = Qt::red;
    c.setAlpha(0);
    QBrush brushel(c);
    p.setBrush(brushel);
    //p.drawEllipse(QRect(QPoint(0,0),this->size()));
    //p.fillRect(0,0,width(),height(),c);
    //qDebug()<< this->geometry();
    p.setRenderHint(QPainter::Antialiasing);
    QPen pen(Qt::black);
    QBrush brush(Qt::white);
    p.setPen(pen);
    p.setBrush(brush);
    //  qDebug()<<"paint :: previousPoints.size"<<previousPoints.size();
    for (int i = 0; i<previousPoints.size();i++)
    {
        QTouchEvent::TouchPoint pt = previousPoints.at(i);

        if (pt.state()!= Qt::TouchPointReleased)
            p.drawEllipse(pt.pos(), 10,10);
        else
            p.drawEllipse(pt.pos(), 3, 3);
    }
}

//-----------------
bool BlobFingerCount::event(QEvent *e)
{
    if (e->type()==QEvent::MouseButtonPress) {
        qDebug()<<"mouse press button";
    }
    if (e->type() == QEvent::TouchBegin) {
        QTouchEvent *te = dynamic_cast<QTouchEvent* >(e);
        touchBegin(te);
        qDebug()<<"touch begin";
        update();
        return true;

    } else if (e->type() == QEvent::TouchUpdate) {
        QTouchEvent *te = dynamic_cast<QTouchEvent* >(e);
        qDebug()<<"touch update";
        touchUpdate(te);
        update();
        return true;

    } else if (e->type() == QEvent::TouchEnd) {
        QTouchEvent *te = dynamic_cast<QTouchEvent* >(e);
        qDebug()<<"touch update";
        touchEnd(te);
        update();
        return true;
    }
    return QWidget::event(e);

}

bool BlobFingerCount::touchBegin(QTouchEvent *e)
{
    QList<QTouchEvent::TouchPoint> points = e->touchPoints();
    previousPoints = points;
    QPoint bary = barycenter(points);
    emit initBarycenter(bary);
    if (points.size() == 1) {
      m_panTimer.start();
    }
    
    return true;
}

bool BlobFingerCount::touchUpdate(QTouchEvent *e)
{
    QList<QTouchEvent::TouchPoint> points = e->touchPoints();
    int nbPoints = points.size();
    qDebug()<<m_right<<"touchUpdate nbPoints "<< nbPoints;
    for (int i=0; i<points.size();i++)
    {
        QTouchEvent::TouchPoint pt = points.at(i);
        if (pt.state() == Qt::TouchPointReleased)
        {
            nbPoints --;
        }
    }
    qDebug()<<m_right<<"touchUpdate nbPoints suite"<< nbPoints;
    if (nbPreviousPoints < nbPoints)
    {
      if (m_panTimer.isActive()) {
	m_panTimer.stop();
	panTimeout();
      }
      if (m_panMode) {
	m_panMode=false;
	emit panStop();
      }
	//qDebug()<<"touchUpdate add point : "<<nbPoints;
        resetComaList();
        QPoint bary = barycenter(points);
        //emit barycenterMoved(bary);
        emit initBarycenter(bary);
        emit nbFingerAdded(nbPoints);
    }
    else if (nbPreviousPoints > nbPoints)
    {
        if (nbPoints == 0)
        {
            if (m_panTimer.isActive()) {
	      m_panTimer.stop(); 
            }
            if (m_panMode) {
	      m_panMode=false;
	      emit panStop();
	    }
	    if (comaList.size()==0) {
                qDebug()<<"Update -> remove ->nbPoints = 0 comalist empty ???????";
                nbPreviousPoints=0;
                previousPoints.clear();
                return true;
            }
	    
            ComaPoint* p = comaList.first();
            if (p)
            {
                int nbFinger = p->nbFingerPrevious();
                resetComaList();
                qDebug()<<"activate from"<<this->m_right<<" Bob::update "<<nbFinger;
                emit activated(nbFinger);
            }
            else
            {

            }
            //previousPoints = e->touchPoints();
            previousPoints.clear();
            return true;
        }
        else
        {
            //qDebug()<<"touchUpdate remove point (previousPoints ="<<this->nbPreviousPoints<<" nbpoints="<<nbPoints<<")";
            ComaPoint* comaPoint = new ComaPoint(nbPreviousPoints, nbPoints,this);
            comaList.append(comaPoint);
        }
    }
    else
    {
        //qDebug()<<"touchUpdate move point";
	if (m_panTimer.isActive()) {
	  m_panTimer.stop();
	  m_panMode = true;
	  emit panStart();
	}
        QPoint bary = barycenter(points);
        emit barycenterMoved(bary);
    }

    previousPoints = points;
    nbPreviousPoints = nbPoints;
    return true;
}

//---------------------
bool BlobFingerCount::touchEnd(QTouchEvent * e)
{
    QList<QTouchEvent::TouchPoint> points = e->touchPoints();
    int nbPoints = points.size();
    //qDebug()<<"touch end: nb points total "<< nbPoints;
    for (int i=0; i<points.size();i++)
    {
        QTouchEvent::TouchPoint pt = points.at(i);
        if (pt.state() == Qt::TouchPointReleased)
        {
            nbPoints --;
        }
    }
    //qDebug()<<"touch end: nb points non release "<< nbPoints;
    if (nbPoints ==0)
    {
      if (m_panMode) {
	m_panMode=false;
	emit panStop();
      }
        if (comaList.size()==0) {
            int nbFinger = points.size();
            resetComaList();
            qDebug()<<"activate from"<<this->m_right<<" BlobFingerCount::touchEnd comalist empty; points.size ="<<nbFinger;
            emit activated(nbFinger);
        } else {
            ComaPoint* p = comaList.first();
            if (p)
            {
                int nbFinger = p->nbFingerPrevious();
                resetComaList();
                qDebug()<<"activate from"<<this->m_right<<" BlobFingerCount::touchEnd comalist not empty"<<nbFinger;
                emit activated(nbFinger);
            }
        }

    }


    //previousPoints = e->touchPoints();
    nbPreviousPoints =0;
    previousPoints.clear();
    return true;
}

//-------------- call from comaPoint
void BlobFingerCount::remove(ComaPoint* comaPoint)
{
    ComaPoint *tmp = comaPoint;
    int nbFinger = comaPoint->nbFinger();
    comaList.removeAll(comaPoint);
    delete tmp;
    //qDebug()<<"BlobFingerCount::remove : removed:"<<nbFinger;
    QPoint bary = barycenter(this->previousPoints);
    //emit barycenterMoved(bary);
    emit initBarycenter(bary);
    emit nbFingerRemoved(nbFinger);
}


//-------------
void BlobFingerCount::resetComaList()
{
    for (int i =0; i< comaList.size();i++)
    {
        delete comaList[i];
    }
    comaList.clear();
}

//--------------
QPoint BlobFingerCount::barycenter(QList<QTouchEvent::TouchPoint>& points)
{
    QPointF res;
    for (int i=0; i<points.size();i++)
    {
        res += points.at(i).pos()+this->pos();
    }
    if (points.size()!=0)
    {
        res/= points.size();
    }

    return res.toPoint();
}

void BlobFingerCount::panTimeout()
{
  emit nbFingerAdded(previousPoints.size());
}


//----------------------------------------
//----------------------------------------
//----------------------------------------
ComaPoint::ComaPoint(int _nbFingerPrevious, int _nbFinger, BlobFingerCount* b):m_nbFinger(_nbFinger),m_nbFingerPrevious(_nbFingerPrevious),m_BlobFingerCount(b) {
    connect(&m_Timer,SIGNAL(timeout()), this, SLOT(timeout()));
    m_Timer.setSingleShot(true);
    m_Timer.start(BlobFingerCount::delayActivate -_nbFinger*5); //qd 2 doigts sont removed pour eviter les conflits
}

//-------------------
void ComaPoint::timeout() {
    //qDebug()<<"timeout";
    m_BlobFingerCount->remove(this);
//qDebug()<<"coma point timeout a corriger";
}

#ifdef USECMAKE
#include "blobfingercount.moc"
#endif
