#include <QDebug>
#include "fctechnique.h"





//----------------------

FCTechnique::FCTechnique()
{
    timerLeftHand.setSingleShot ( true );
    connect ( &timerLeftHand,SIGNAL ( timeout() ),this,SLOT ( timerLeftHandFinished() ) );
    timerExpert.setSingleShot ( true );
    connect ( &timerExpert,SIGNAL ( timeout() ), this, SLOT ( timerExpertFinished() ) );

    nbRightHandFinger = nbLeftHandFinger = 0;
    nbRightHandFingerExpert = nbLeftHandFingerExpert = 0;
    state = CLOSED;
    expertModeDelay = 450;
    leftHandRemoveDelay = 350;
    secondaryZone = 5;
    continuousZone = 15;
    
}

//-----------------------
void FCTechnique::setBlobsFC ( BlobFingerCount *_left, BlobFingerCount *_right )
{
    connect ( _left,SIGNAL ( nbFingerAdded ( int ) ), this, SLOT ( addLeftHandFinger ( int ) ) );
    connect ( _left, SIGNAL ( nbFingerRemoved ( int ) ), this, SLOT ( removeLeftHandFinger ( int ) ) );
    connect ( _left, SIGNAL ( activated ( int ) ), this, SLOT ( activateLeftHandFinger ( int ) ) );
    connect ( _left,SIGNAL ( barycenterMoved ( const QPoint& ) ),this,SLOT ( moveLeftHand ( const QPoint&) ) );
    connect ( _left,SIGNAL ( initBarycenter ( const QPoint& ) ), this, SLOT ( resetLeftHand ( const QPoint& ) ) );
    
    connect ( _left,SIGNAL(panStart()),this,SIGNAL(panActionStart()));
    connect ( _left,SIGNAL(panStop()),this,SIGNAL(panActionStop()));

    connect ( _right,SIGNAL ( nbFingerAdded ( int ) ), this, SLOT ( addRightHandFinger ( int ) ) );
    connect ( _right, SIGNAL ( nbFingerRemoved ( int ) ), this, SLOT ( removeRightHandFinger ( int ) ) );
    connect ( _right, SIGNAL ( activated ( int ) ), this, SLOT ( activateRightHandFinger ( int ) ) );
    connect ( _right,SIGNAL ( barycenterMoved ( QPoint) ),this,SLOT ( moveRightHand ( QPoint) ) );
    connect ( _right,SIGNAL ( initBarycenter ( const QPoint& ) ), this, SLOT ( resetRightHand ( const QPoint& ) ) );

    connect ( _right,SIGNAL(panStart()),this,SIGNAL(panActionStart()));
    connect ( _right,SIGNAL(panStop()),this,SIGNAL(panActionStop()));
}

//-----------------------
void FCTechnique::setBlobsRS ( BlobRadialStroke *_left, BlobRadialStroke *_right )
{
    connect ( _right,SIGNAL ( stroke1Changed ( int ) ), this, SLOT ( addLeftHandFinger ( int ) ) );

    connect ( _right, SIGNAL ( stroke1Activated ( int ) ), this, SLOT ( activateLeftHandFinger ( int ) ) );
    connect ( _left,SIGNAL ( barycenterMoved ( const QPoint& ) ),this,SLOT ( moveLeftHand ( const QPoint& ) ) );
    connect ( _left,SIGNAL ( initBarycenter ( const QPoint& ) ), this, SLOT ( resetLeftHand ( const QPoint& ) ) );
    connect ( _left, SIGNAL ( menuClosed() ), this, SLOT ( closeMenu() ) );

    connect ( _right,SIGNAL ( stroke2Changed ( int ) ), this, SLOT ( addRightHandFinger ( int ) ) );
    connect ( _right, SIGNAL ( stroke2Activated ( int ) ), this, SLOT ( activateRightHandFinger ( int ) ) );
    connect ( _right,SIGNAL ( barycenterMoved ( QPoint ) ),this,SLOT ( moveRightHand ( QPoint ) ) );
    connect ( _right,SIGNAL ( initBarycenter ( const QPoint& ) ), this, SLOT ( resetRightHand ( const QPoint& ) ) );


}

//-----------------------
void FCTechnique::setFinishExpertMode()
{
    qDebug() <<__PRETTY_FUNCTION__<<"lexpert="<<nbLeftHandFingerExpert <<" rexpert="<<nbRightHandFingerExpert;
    //qDebug()<<__PRETTY_FUNCTION__<<"lnormal="<<nbLeftHandFinger <<" rnormal="<<nbRightHandFinger;

    timerExpert.stop();
    timerExpertFinished();
}

//-----------------------
void FCTechnique::timerExpertFinished()
{
    if ( nbLeftHandFinger == 0 && nbRightHandFinger == 0 )
    {
        qDebug() <<"selection in expert mode l="<<nbLeftHandFingerExpert <<" r="<<nbRightHandFingerExpert;
        activateItem ( nbLeftHandFingerExpert, nbRightHandFingerExpert, secState );
        curIdMenu = curIdItem = 0;
        setSecState(DIRECT);
        setState ( CLOSED );
    }
    else
    {
        qDebug() <<"selection in novice mode l="<<nbLeftHandFinger <<" r="<<nbRightHandFinger;

        qDebug() <<"selection in novice mode le="<<nbLeftHandFingerExpert <<" re="<<nbRightHandFingerExpert;
        setState ( NOVICE );
        selectItem ( nbLeftHandFinger, nbRightHandFinger,secState );
    }
    nbLeftHandFingerExpert = nbRightHandFingerExpert = 0;
}

//--------------------------
void FCTechnique::timerLeftHandFinished()
{
    nbLeftHandFinger = 0;
    selectMenu ( 0 );
    setState ( CLOSED );
}

//---------------------
void FCTechnique::addLeftHandFinger ( int nbF )
{
    if ( state == CLOSED )
    {
        nbLeftHandFingerExpert = nbLeftHandFinger = nbF;
        timerExpert.start ( expertModeDelay );
        setState ( EXPERT );
    }
    else if ( state == EXPERT )
    {
        timerExpert.start ( expertModeDelay );
        nbLeftHandFinger = nbF;
        setSecState(DIRECT);
        if ( nbF > nbLeftHandFingerExpert )
        {
            //qDebug()<<" FCTechnique::addLeftHandFinger -> nbLeftHandFingerExpert "<<nbF;
            nbLeftHandFingerExpert = nbF;
        }
    }
    else if ( state == NOVICE )
    {
        timerLeftHand.stop();
        selectMenu ( nbF );
        setSecState(DIRECT);
    }
    else if ( state == CONFIRM )
    {
    }
}

//-----------------------------------
void FCTechnique::removeLeftHandFinger ( int nbF )
{
    if ( state == CLOSED )
    {
        qDebug() <<"error : state CLOSED -> removeLeftHandFinger("<<nbF<<")";
    }
    else if ( state == EXPERT )
    {
        nbLeftHandFinger = nbF;
        setSecState(DIRECT);
    }
    else if ( state == NOVICE )
    {
        selectMenu ( nbF );
        setSecState(DIRECT);
        if ( nbF == 0 && curIdItem == 0 )
        {
            setState ( CLOSED );
        }
    }
    else if ( state == CONFIRM )
    {
        qDebug() <<"error : state CONFIRM -> removeLeftHandFinger("<<nbF<<")";
    }

}

//--------------------------
void FCTechnique::closeMenu()
{
    reset();
    setState ( CLOSED );
}

//-----------------------------------
void FCTechnique::activateLeftHandFinger ( int nbF )
{

    if ( state == CLOSED )
    {
        qDebug() <<"warning : state CLOSED -> activateFingerShortcutNDH("<<nbF<<")";
    }
    else if ( state == EXPERT )
    {
//        if(nbF > nbLeftHandFinger )
//        {
//            nbLeftHandFinger = nbF;
//            timerExpert.start(expertModeDelay);
//        }
        nbLeftHandFinger = 0;
        if ( nbLeftHandFingerExpert!=0 && nbRightHandFingerExpert!=0 && nbLeftHandFinger==0 && nbRightHandFinger==0 )
        {
            setFinishExpertMode();
        }

        //on ne relance pas le timer
    }
    else if ( state == NOVICE )
    {
        qDebug() <<"fc technique activate left hand";
        if ( curIdItem == 0 )
        {
            qDebug() <<"curIdItem ==0";
            reset();
            setState ( CLOSED );
        }
        else
        {
            qDebug() <<"lancement timer curIdItem !=0";
            nbLeftHandFinger = nbF;
            timerLeftHand.start ( leftHandRemoveDelay ); //laisse un peu de temps Ã  l'autre main pour activer l'item
        }

    }
    else if ( state == CONFIRM )
    {
        qDebug() <<"error : state CONFIRM -> activateFingerShortcutNDH("<<nbF<<")";
    }
}

//--------------------------
void FCTechnique::addRightHandFinger ( int nbF )
{
    if ( state == CLOSED )
    {
        nbRightHandFingerExpert = nbRightHandFinger = nbF;
        timerExpert.start ( expertModeDelay );
        setSecState(DIRECT);
        setState ( EXPERT );
    }
    else if ( state == EXPERT )
    {
        nbRightHandFinger = nbF;
        if ( nbF > nbRightHandFingerExpert )
        {
            nbRightHandFingerExpert = nbF;
        }
        timerExpert.start ( expertModeDelay );
    }
    else if ( state == NOVICE )
    {
        nbRightHandFinger = nbF;
        setSecState(DIRECT);
        selectItem ( nbRightHandFinger,secState );
    }
    else if ( state == CONFIRM )
    {
        qDebug() <<"error : state CONFIRM -> fingerShortcutDHAdd("<<nbF<<")";
    }

}

//-------------------------
void FCTechnique::removeRightHandFinger ( int nbF )
{
    if ( state == CLOSED )
    {
        qDebug() <<"error : state CLOSED -> fingerShortcutDHRemove("<<nbF<<")";
    }
    else if ( state == EXPERT )
    {
        setSecState(DIRECT);
        nbRightHandFinger = nbF;
    }
    else if ( state == NOVICE )
    {
        nbRightHandFinger = nbF;
setSecState(DIRECT);
        selectItem ( nbRightHandFinger,secState );
    }
    else if ( state == CONFIRM )
    {
        qDebug() <<"error : state CONFIRM -> fingerShortcutDHRemove("<<nbF<<")";
    }
}

//--------------------------
void FCTechnique::activateRightHandFinger ( int nbF )
{
    if ( state == CLOSED )
    {
        qDebug() <<"warning : state CLOSED -> activateFingerShortcutDH("<<nbF<<")";
    }
    else if ( state == EXPERT )
    {
        nbRightHandFinger = 0;
        if ( nbLeftHandFingerExpert!=0 && nbRightHandFingerExpert!=0 && nbLeftHandFinger==0 && nbRightHandFinger==0 )
        {
            setFinishExpertMode();
        }
        //if(nbF > nbRightHandFinger)
        //{
        //    nbRightHandFinger = nbF;
        //    timerExpert.start(expertModeDelay);
        //}

    }
    else if ( state == NOVICE )
    {
        if ( timerLeftHand.isActive() )
        {
            //timerLeftHand.stop();
            activateItem ( nbLeftHandFinger, nbF,secState );
            //nbLeftHandFinger = 0;
            nbRightHandFinger = 0;
        setSecState(DIRECT);
            //selectItem(nbLeftHandFinger, 0);
            //setState(CLOSED);
        }
        else
        {
            activateItem ( nbF,secState );
           setSecState(DIRECT);
            nbRightHandFinger = 0;
            selectCurAction();
            //selectItem(nbLeftHandFinger, nbRigthHandFinger);
            // activateItem(0);
        }
    }

    else if ( state == CONTROL) {
        activateItem (nbF, secState);
       setSecState(DIRECT);
        nbRightHandFinger = 0;
        selectCurAction();
    }

    else if ( state == CONFIRM )
    {

    }
}

//-------------------
void FCTechnique::resetLeftHand ( const QPoint& p )
{
    leftPos.setP1 ( p );
    leftPos.setP2 ( p );
    emit this->handPositionInitialized ( leftPos, rightPos );
}

//-------------------
void FCTechnique::resetRightHand ( const QPoint& p )
{
    rightPos.setP1 ( p );
    rightPos.setP2 ( p );
    emit this->handPositionInitialized ( leftPos, rightPos );
}

//-------------------
void FCTechnique::moveLeftHand ( const QPoint& p)
{
    leftPos.setP2 ( p );
    updateHandsPosition ();
}

//-------------------
void FCTechnique::moveRightHand ( const QPoint& p)
{
    rightPos.setP2 ( p );
    
      //  if ( rightPos.dx() > secondaryZone && rightPos.dx() < continuousZone )
    if ( secondaryZone < rightPos.dx() && rightPos.dx() < continuousZone )
    {
            //Up
          setSecState(UP);
          selectItem (nbRightHandFinger,secState);
            qDebug() << "Select previous action";

                    }
        else if ( -secondaryZone > rightPos.dx() && rightPos.dx() > -continuousZone )
        {
            //Down
            setSecState(DOWN);
            selectItem (nbRightHandFinger,secState);
            qDebug() << "Select next action";

        }
        else if ( abs(rightPos.dx() < secondaryZone)) {
            setSecState(DIRECT);
            selectItem (nbRightHandFinger,secState);
        }
        else if ( abs(rightPos.dx()) > continuousZone || abs(rightPos.dy()) > secondaryZone)
        {
            //Center
            //setSecState(DIRECT);
            selectItem ( nbRightHandFinger, secState, true );
        }
    //selectItem ( nbRightHandFinger, secState );
    selectItem(secState);
    updateHandsPosition ();
}

//-------------------
void FCTechnique::updateHandsPosition ()
{
    //if ( !fingerAdd )
    //{
        if ( abs ( leftPos.dx() ) > continuousZone
                || abs ( rightPos.dx() ) > continuousZone
                || abs ( leftPos.dy() ) > continuousZone
                || abs ( rightPos.dy() ) > continuousZone )
        {
            //if (abs(leftPos.dx())+abs(leftPos.dy()) > 20 || abs(rightPos.dx())+abs(rightPos.dy()) > 20)
            //state = CONTROL;
            qDebug() << "State : CONTROL";
            selectItem ( nbRightHandFinger, secState, true );
        }
    //}

    emit handMoved ( leftPos, rightPos);

}


#ifdef USECMAKE
#include "fctechnique.moc"
#endif





