/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#ifndef MINIFINGERCOUNT_H
#define MINIFINGERCOUNT_H

#include <QObject>
#include <QMap>
#include <QAction>
#include "blobfingercount.h"

/**
\class MiniFingerCount
\brief Finger-Count Shortcuts popu menu version
This class provide a simple version of the finger-count shortcuts executing commands from a simple list
@author Gilles Bailly, Alexandre Viard
*/
class MiniFingerCount : public QObject
{
  Q_OBJECT
  
  public:
    /**
     * Default Constructor
     */
    MiniFingerCount(QObject* parent);
    
    /**
    * Default Destructor
    */
    virtual ~MiniFingerCount();
    
    /**
     * Add a new action to the Finger-Count list, if an action with a number already use will replace the previous
     * \param act the action to add
     * \param fc the number of fingers needed to trigger it
     */
    virtual QAction* addAction(QAction* act, int fc);
    
    /**
     * Set the Widget that would provide the number of fingers on its surface
     * \param _blob the widget receiving the touch event
     */
    virtual void setBlob(BlobFingerCount* _blob);
    
  public slots :
    /**
     * Called when fingers are detected
     * \param nba the number of fingers
     */
    virtual QAction* selectAction(int nba);
    
    /**
    * Called when the activation gesture is detected
    * \param nba the number of fingers
    */
    virtual QAction* activateAction(int nba);
    
    /**
    * Called when more fingers are detected
     * \param nbf number of fingers
     */
    void addHandFinger(int);
    
    /**
    * Called when less fingers are detected
    * \param nbf the number of fingers
    */
    void removeHandFinger(int);
    
    /**
    * Called when the hand is removed from the surface
    * \param nbf the number of fingers
    */
    void activateHandFinger(int);
    
    /**
    * Triggered when the hand move on the surface
    * \param p the new position 
    */
    void moveHand(const QPoint& p);
    
    /**
    * Send the initial left hand position
     * \param p the initial position 
     */
    void resetHand(const QPoint& p);
    
  signals:
    /**
     * Provide the information on the currently selected command
     */
    void itemSelected(int nbF);
    
  protected:
  QMap< int, QAction*> fingerAction;
  BlobFingerCount* blob;
  int currentFinger;
  
  
};

#endif // MINIFINGERCOUNT_H
